var config = {
  "colors": [],
  "image": {"data": {}},
  "format": {
    "size": function (s) {
      if (s) {
        if (s >= Math.pow(2, 30)) return (s / Math.pow(2, 30)).toFixed(1) + ' ' + "GB";
        else if (s >= Math.pow(2, 20)) return (s / Math.pow(2, 20)).toFixed(1) + ' ' + "MB";
        else if (s >= Math.pow(2, 10)) return (s / Math.pow(2, 10)).toFixed(1) + ' ' + "KB";
        else return s + ' ' + 'B';
      } else return "0.00" + ' ' + 'B';
    }
  },
  "standard": {
    "time": function (secs) {
      if (secs > 0) {
        secs = secs / 1000;
        var H = Math.floor(Math.floor(secs / 3600) % 24);
        var M = Math.floor(Math.floor(secs / 60) % 60);
        var S = Math.floor(secs % 60);
        return [H, M, S].map(e => e < 10 ? '0' + e : e).filter((v, i) => v !== "00" || i > 0).join(':');
      } else return "00:00:00";
    }
  },
  "file": {
    "extension": function (e) {
      return e ? (/\.(.*)/.exec(e) ? /\.(.*)/.exec(e)[0] : e) : ".unknown";
    },
    "name": function (e) {
      return e ? e.substring(Math.max(e.lastIndexOf('\\'), e.lastIndexOf('/')) + 1) : "unknown";
    },
    "icon": function (e) {
      var table = document.querySelector("table[data-id='" + e.id + "']");
      if (table) {
        var image = table.querySelector(".image");
        if (image) {
          var icon = image.querySelector(".icon");
          if (icon) {
            var cache = config.image.data[e.fileextension];
            if (cache) icon.setAttribute("src", cache);
            else {
              if (e.src.indexOf("unknown.png") === -1) {
                config.image.data[e.fileextension] = e.src;
                icon.setAttribute("src", config.image.data[e.fileextension]);
              }
            }
          }
        }
      }
    }
  },
  "load": function (e) {
    if (e.reset) document.querySelector(".items").textContent = '';
    if (e.listener) {
      var add =  document.getElementById("add");
      var open =  document.getElementById("open");
      var erase = document.getElementById("erase");
      var order = document.getElementById("order");
      var addurl = document.getElementById("addurl");
      var thread = document.getElementById("thread");
      var refresh = document.getElementById("refresh");
      var support = document.getElementById("support");
      var maxitems = document.getElementById("maxitems");
      var donation = document.getElementById("donation");
      var input = document.getElementById("search-input");
      var search = document.getElementById("search-button");
      /*  */
      open.onclick = function () {background.send("folder")};
      erase.onclick = function () {background.send("erase")};
      refresh.onclick = function () {document.location.reload()};
      support.onclick = function () {background.send("support")};
      donation.onclick = function () {background.send("donation")};
      add.onclick = function () {background.send("download", addurl.value)};
      thread.onchange = function (e) {background.send("thread", parseInt(e.target.value))};
      maxitems.onchange = function (e) {background.send("maxitems", parseInt(e.target.value))};
      input.onkeyup = function (e) {background.send("search", {"query": input.value.split(' ')})};
      search.onclick = function (e) {background.send("search", {"query": input.value.split(' ')})};
      order.onchange = function (e) {background.send("orderBy", e.target[e.target.selectedIndex].value)};
      /*  */
      addurl.value = e.data.url;
      order.value = e.data.orderBy;
      thread.value = e.data.threads;
      config.colors = e.data.colors;
      maxitems.value = e.data.maxitems;
    }
    /*  */
    for (var i = 0; i < e.data.downloads.length; i++) {
      var addlistener = false;
      var item = e.data.downloads[i];
      if (item) {
        var valid = item.startTime;
        if (valid) {
          var items = document.querySelector('.items');
          var table = items.querySelector("table[data-id='" + item.id + "']");
          if (!table) {
            var template = document.querySelector('#template').content;
            try {
              table = template.querySelector("table").cloneNode(true);
              table.setAttribute("class", "item");
              table.setAttribute("data-id", item.id);
              if ("sections" in item) table.setAttribute("multi", true);
              items.appendChild(table);
              addlistener = true;
            } catch (e) {}
          }
          /*  */
          config.render(item, e.data.threads, e.progress, addlistener);
        }
      }
    }
  },
  "render": function (item, threads, progress, addlistener) {
    var table = document.querySelector("table[data-id='" + item.id + "']");
    if (table) {
      var name = table.querySelector(".name");
      var info = table.querySelector(".info");
      var fileextension = table.getAttribute("fileextension");
      var urlinfo = name ? name.querySelector(".urlinfo") : undefined;
      var fileinfo = name ? name.querySelector('.fileinfo') : undefined;
      var speedinfo = info ? info.querySelector('.speedinfo') : undefined;
      var left = speedinfo ? speedinfo.querySelector('.left') : undefined;
      var speed = speedinfo ? speedinfo.querySelector('.speed') : undefined;
      var elapsed = speedinfo ? speedinfo.querySelector('.elapsed') : undefined;
      var downloadinfo = name ? name.querySelector(".downloadinfo") : undefined;
      var size = downloadinfo ? downloadinfo.querySelector(".size") : undefined;
      var time = downloadinfo ? downloadinfo.querySelector(".time") : undefined;
      var filetitle = fileinfo ? fileinfo.querySelector('.filename') : undefined;
      var state = downloadinfo ? downloadinfo.querySelector(".state") : undefined;
      var received = speedinfo ? speedinfo.querySelector('.progress') : undefined;
      /*  */
      if (left) left.textContent = "Left: 00:00";
      if (speed) speed.textContent = "Speed: 0.00 KB/s";
      if (elapsed) elapsed.textContent = "Elapsed: 00:00";
      if (received) received.textContent = "Received: 0.00B of 0.00B";
      if (size) size.textContent = item.totalBytes > 0 ? config.format.size(item.totalBytes) : "0.00B";
      if (time) time.textContent = item.startTime ? (new Date(item.startTime)).toLocaleTimeString() : "00:00:00";
      /*  */
      if (state) {
        if (item.state === "complete") state.textContent = "complete";
        if (item.state === "transfer") state.textContent = "preparing";
        if (item.state === "in_progress") state.textContent = item.threads ? (item.threads > 1 ? item.threads + " threads" : "1 thread") : "...";
      }
      /*  */
      if (fileextension) {
        if (config.image.data[fileextension]) {
          var image = table.querySelector(".image");
          if (image) {
            var icon = image.querySelector(".icon");
            icon.setAttribute("src", config.image.data[fileextension]);
          }
        }
      }
      /*  */
      if (urlinfo) {
        var url = urlinfo.querySelector(".url");
        if (url) {
          url.setAttribute("href", item.url);
          var filename = item.filename;
          if (filename) {
            var target = urlinfo.querySelector("td");
            if (target) {
              target.setAttribute("state", item.state);
              if (item.state === "in_progress") {
                var tmp = {"size": '', "image": '', "repeat": "no-repeat"};
                if (item.sections && item.sections.length) {
                  for (var i = 0; i < item.sections.length; i++) {
                    if (item.sections[i]) {
                      tmp.size = (tmp.size ? tmp.size + ", " : '') + ((item.sections[i][1] / item.totalBytes) * 100 + '%' + " 100%");
                      tmp.image = (tmp.image ? tmp.image + ", " : '') + "linear-gradient(" + config.colors[i] + ', ' + config.colors[i] + ')';
                      /*  */
                      if (item.sections[i + 1]) {
                        if (item.sections[i][1] !== item.sections[i + 1][0]) {
                          tmp.image = (tmp.image ? tmp.image + ", " : '') + "linear-gradient(#fff, #fff)";
                          tmp.size = (tmp.size ? tmp.size + ", " : '') + ((item.sections[i + 1][0] / item.totalBytes) * 100 + '%' + " 100%");
                        }
                      }
                    }
                  }
                } else {
                  tmp.image = (tmp.image ? tmp.image + ", " : '') + "linear-gradient(#239200, #239200)";
                  tmp.size = (tmp.size ? tmp.size + ", " : '') + ((item.bytesReceived / item.totalBytes) * 100 + '%' + " 100%");
                }
                /*  */
                target.removeAttribute("state");
                target.style.backgroundSize = tmp.size;
                target.style.backgroundImage = tmp.image;
                target.style.backgroundRepeat = tmp.repeat;
              }
            }
          }
        }
      }
      /*  */
      if (speedinfo) {
        if (elapsed) {
          var estimated = (new Date()).getTime() - (new Date(item.startTime)).getTime();
          elapsed.textContent = "Elapsed: " + (estimated ? config.standard.time(estimated) : "00:00");
        }
        /*  */
        if (received) {
          var total = item.totalBytes > 0 ? config.format.size(item.totalBytes) : "0.00B";
          received.textContent = "Received: " + config.format.size(item.bytesReceived) + ' of ' +	total;
        }
        /*  */
        if (left) {
          var timeleft = null;
          if (item.endTime) {
            timeleft = new Date(item.endTime).getTime() - (new Date()).getTime();
          } else if (item.estimatedEndTime) {
            timeleft = new Date(item.estimatedEndTime).getTime() - (new Date()).getTime();
          } else if (item.startTime && item.speed) {
            var estimatedEndTime = (new Date(item.startTime)).getTime() + (item.totalBytes / item.speed) * 1000;
            timeleft = new Date(estimatedEndTime).getTime() - (new Date()).getTime();
          }
          /*  */
          left.textContent = "Left: " + (timeleft && timeleft > 0 ? config.standard.time(timeleft) : "00:00");
        }
        /*  */
        if (speed) {
          if (item.speed) {
            speed.textContent = "Speed: " + config.format.size(item.speed) + "/s";
          } else {
            var startTime = item.startTime ? new Date(item.startTime).getTime() : null;
            var endTime = item.estimatedEndTime ? new Date(item.estimatedEndTime).getTime() : null;
            var duration = endTime && startTime ? (endTime - startTime) / 1000 : 0;
            var Bps = duration ? ((item.totalBytes ? item.totalBytes : 0) / duration).toFixed(2) : 0;
            var show = item.paused === false && Bps && !isNaN(parseFloat(Bps)) && isFinite(Bps);
            var result = show ? config.format.size(Bps) + "/s" : "0.00 KB/s";
            speed.textContent = "Speed: " + result;
          }
        }
      }
      /*  */
      if (progress === false) {
        var filename = config.file.name(item.filename);
        var fileextension = config.file.extension(filename);
        if (filename) table.setAttribute("filename", filename);
        if (fileextension) table.setAttribute("fileextension", fileextension);
        /*  */
        if (fileinfo) {
          var error = fileinfo.querySelector(".error");
          if (item.error) {
            name.setAttribute("disabled", true);
            if (error) error.textContent = item.error.toLowerCase();
          } else {
            error.textContent = '';
            name.removeAttribute("disabled");
          }
        }
        /*  */
        if (filetitle) {
          filetitle.textContent = table.getAttribute("filename");
          if (addlistener) {
            filetitle.onclick = function () {
              background.send("open", {"id": item.id});
            };
          }
          /*  */
          var image = table.querySelector(".image");
          if (image) {
            var icon = image.querySelector(".icon");
            if (icon) {
              var src = icon.getAttribute("src");
              if (src) {
                if (src.indexOf("unknown.png") !== -1) {
                  background.send("icon", {
                    "id": item.id,
                    "multi": ("sections" in item),
                    "fileextension" : fileextension
                  });
                }
              }
            }
          }
        }
        /*  */
        if (downloadinfo) {
          var size = downloadinfo.querySelector(".size");
          var time = downloadinfo.querySelector(".time");
          var pause = downloadinfo.querySelector(".pause");
          var erase = downloadinfo.querySelector(".erase");
          var resume = downloadinfo.querySelector(".resume");
          var cancel = downloadinfo.querySelector(".cancel");
          var remove = downloadinfo.querySelector(".remove");
          var referrer = downloadinfo.querySelector(".referrer");
          var openfolder = downloadinfo.querySelector(".openfolder");
          /*  */
          item.exists = navigator.userAgent.indexOf("Firefox") !== -1 ? true : item.exists;
          /*  */
          if (resume) {
            resume.setAttribute("disabled", true);
            resume.setAttribute("data-id", item.id);
            if (addlistener) {
              resume.onclick = function () {
                background.send("resume", {
                  "multi": "sections" in item,
                  "id": parseInt(this.getAttribute("data-id"))
                });
              };
            }
            /*  */
            if (item.exists && item.paused) {
              var flag_1 = item.state === "interrupted";
              var flag_2 = item.state === "in_progress";
              if (flag_1 || flag_2) resume.removeAttribute("disabled");
            }
          }
          /*  */
          if (pause) {
            pause.setAttribute("disabled", true);
            pause.setAttribute("data-id", item.id);
            if (addlistener) {
              pause.onclick = function () {
                background.send("pause", {
                  "multi": "sections" in item,
                  "id": parseInt(this.getAttribute("data-id"))
                });
              };
            }
            /*  */
            if (item.exists && !item.error) {
              if (item.paused === false) {
                if (item.state === "in_progress") {
                  pause.removeAttribute("disabled");
                }
              }
            }
          }
          /*  */
          if (cancel) {
            cancel.setAttribute("disabled", true);
            cancel.setAttribute("data-id", item.id);
            if (addlistener) {
              cancel.onclick = function () {
                background.send("cancel", {
                  "multi": "sections" in item,
                  "id": parseInt(this.getAttribute("data-id"))
                });
              };
            }
            if (item.exists && !item.error) {
              var flag_1 = item.state === "in_progress";
              var flag_2 = item.state === "interrupted";
              if (flag_1 || flag_2) cancel.removeAttribute("disabled");
            }
          }
          /*  */
          if (openfolder) {
            openfolder.setAttribute("disabled", true);
            openfolder.setAttribute("data-id", item.id);
            if (item.exists) openfolder.removeAttribute("disabled");
            if (addlistener) openfolder.onclick = function () {background.send("folder")};
          }
          /*  */
          if (erase) {
            erase.setAttribute("data-id", item.id);
            if (addlistener) {
              erase.onclick = function () {
                background.send("erase", {"id": parseInt(this.getAttribute("data-id"))});
              };
            }
          }
          /*  */
          if (remove) {
            remove.setAttribute("disabled", true);
            remove.setAttribute("data-id", item.id);
            if (addlistener) {
              remove.onclick = function () {
                var id = parseInt(this.getAttribute("data-id"));
                mscConfirm("Delete", "Are you sure you want to delete this item from disk?", function () {
                  background.send("remove", {"id": id});
                });
              };
            }
            if (item.exists) {
              if (item.state === "complete") {
                remove.removeAttribute("disabled");
              }
            }
          }
          /*  */
          if (referrer) {
            referrer.setAttribute("disabled", true);
            referrer.setAttribute("data-id", item.id);
            if (addlistener) {
              referrer.onclick = function () {
                var tmp = this.getAttribute("referrer");
                if (tmp) background.send("referrer", tmp);
              }
            }
            if (item.referrer) {
              referrer.removeAttribute("disabled");
              referrer.setAttribute("referrer", item.referrer);
            }
          }
        }
      }
    }
  }
};
